#ifndef SW_GLOBALS
#define SW_GLOBALS
#include <stdio.h>   
#include <limits.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h> /* For erasing cursor - not important */
#include <math.h>
#ifdef SOLARIS
#include <string.h>
#else
#include <strings.h>
#endif



#define shipstatusy  200

/* Indexes for 1st dimension of obj */
/* The order they are in is important */

#define FIRSTPLAYER 1
#define MAXPLAYERS 10
#define DEFAULTSHIPSIZE 28
/* if max players is bumped up AST must also be increased so the player does 
   not become an asteriod */
#define AST 11
#define LASTAST 35
#define ENEMY 9
#define ENEMYBUL 36
#define FBUL  37
#define LASTBUL 105

#define LASTOBJ 105


/* Shapes */
/* Order is important!  See collide().  */
/*# fix the order !!! */
#define ASTSHAPE1 0
#define ASTSHAPE2 1
#define ASTSHAPE3 2
#define ENBULSH   3
#define BULSHAPE  4
#define SHIPSHAPE 5
#define SHIPTHRSHAPE  6
#define SHIPSHAPE2  7
#define SHIPTHRSHAPE2 8
#define LASTSHAPE 25


#define BMAX    200 /* Max particles in a "boom" + 1  */

#define pi    3.1415926535897932384

#define NUMSYSTEMS 5
#define cloaksys 0
#define shieldsys 1
#define enginesys 2
#define weaponsys 3
#define hullsys 4

#define turretsys 5
#define rotationsys 6
#define energysys 7

#define DESTROYED 0
#define OFFLINE 1
#define DAMAGED 2
#define ONLINE 3


#define NOSHIP -1

typedef struct _Boom *Boom;
struct _Boom {Boom   next; 
              int    dur;
              int    part; 
              double bcoord[BMAX][2];
              double bvec[BMAX][2];};

/* The SystemType structure will contain all of the data that is common across 
    all of the systems that make up a ship.  The ship systems include : cloak, 
    sheild, engine, weapon,hull, turret, rotation and energy */

typedef struct { 
       /* this keeps track of the amount of damage the system has. If hitpoints
          goes below 0 the system is destroyed */
   int hitpoints;

        /* This holds the maximum hit points for a system.  If hitspoints =
            maxhitpoints there is no damage to the system */
   int maxhitpoints;

       /* This holds the number of turns needed to repair one hitpoint on the
           system. */
   int repairrate;

       /* This is incremented every turn by one.  If the repair counter = repair
           rate then the system will gain one hitpoint (if it is damaged)*/
   int repaircounter;  

       /* Holds if the system is on line, off line, damaged or destroyted */
   int status;
               } SystemType;

/* The ShipType structure contains all of the data that is needed
   to define a ship. Note that this is an extention of an object record and 
   each ship record has an associated object record. */

typedef struct {
        /* Amount of turns requried to elapse before a weapon can fire again */
  int   reloadtime;     
         /* Current number of turns since last fire */
  int   reload;
        /* This is part of the caclulation for how long a bullet will appear on
           the screen.  The lower the value the longer the bullet will last */
  int   bulletduration;
        /* Max amount of energy that a ship can have */
  int   initialenergy;
        /* Amount of energy consumed when shields are activated */
  int   shieldenergy;
        /* Amount of energy consumed when shields are hit by an object */
  int   shieldhitenergy;
        /* Amount of energy consumed when thrust system is activated */
  int   thrustenergy;
        /* Amount of energy consumed when rotation system is activated */
  int   rotateenergy;
        /* Amount of energy consumed when weapons are fired */
  int   fireenergy;
        /* Amount of energy consumed when super weapon is fired*/
  int   superfireenergy;
        /* Amount of energy consumed when hyper space is activated*/
  int   hyperenergy;
        /* Amount of energy consumed when cloak system is activated */
  int   cloakenergy;
        /* Amount of energy regenerated every turn */
  int   regenerateenergy;
        /* Number of bullets contained in the super weapon */
  int   numsuperfire;
        /* Number of projectiles when ship self destructs */
  int   numselfdestruct;
        /* The diameter (in virutal coordinates) of the ship. */
  int   size;
        /* Amount of damage a bullet fired by this ship causes */
  int   damage;
        /* Is set to 1 if this ship is allowed to fire in any direction */
  int   hasturret;
        /* Amount of damage the shields can aborb when hit */
  int   shieldstrength;
        /* Indicates if the shield system is activated */
  int   shield_on;
        /* Indicates if the cloak system is activated */
  int   cloak_on;
        /* Indicates if the thrust system is activated */
  int   thrust_on;
        /* Number of ships destroyed by the owner of this ship*/
  int   kills;
        /* Number of times the owner of this ship got destroyed */
  int   killed;
        /* The ship that this ship is currently scanning */
  int   scannedship;
        /* This is the speed of a bullet fired from this ship */
  float bulletvelocity; 
        /* # of radians per turn the ship can be rotated */
  float rotationincrement;
        /* Maximum coasting speed of a ship */
  float maxvelocity;
        /* Maximum speed of a ship while thurst is on */
  float maxthrustvelocity;
        /* Accelaration of the ship */
  float accelaration;

  SystemType system[NUMSYSTEMS];} /* All of the stystem in the ship */
       ShipType;



/* The objtype structure contains all of the data requried for displayable
   objects.  Displayable objects include : bullets, planets and ships */
typedef struct {
  int    shape;  /* Index into the shape array for the shape of this object */
  int    alive;  /* Indicates if this object is active and viewable on the screen */
  int    time;   /* A timer for the object that is decremented after every turn */
  double x;
  double y;   /* Position of the object */ 
  double xvel;
  double yvel; /* Velocity of the object */
  double rot;  /* Current rotation angle of the object */
  double rotvel; /* Current rotation velocity of the object */
  double mass;   /* Mass of the object */
  int    blowingup; /* This indicates if the object is blowing up */
  int    owner; /* The owner of the object */
  int    energy; }/* Amount of engergy the object contains */

       Objtype;

typedef struct {double angle; 
                int    length;} PolarPair;
typedef struct {int x;
                int y;} XYPair;

typedef struct {double x;
                double y;
                double mag;} Vector;



/* Defines the entire domain of the simulation.  No objects can go outside
   of the domain of the simulation */
typedef struct {int x1;
                int y1;
                int x2;
                int y2;
                int width;
                int height;} VirtualType;
/* Defines the entire domain of the simulation */
/* x1,y1 is the upper left corner and x2,y2 is the lower right corner */


/* Defines the window into the domain that will appear on window shown to the
    user */
typedef struct {int x;
                int y;
                int width;
                int height;} PhysicalType;

/* Actual size of the window shown to the user */
typedef struct {
     int width;
     int height;
     double scalex;
     double scaley;
     double longrangescalex;
     double longrangescaley;} 
 DeviceType;



typedef struct _Xparam *Xparam;

/* The _Xparam record contains all of the data needed for every display device in
   the simulation */
struct _Xparam { 
     Display       *disp; /* The X display connected to */
     Pixmap        pixmap;/* The pixmap used for double buffering (DB)*/
     Pixmap        lrpixmap; /* The pixmap used for DB of long range scan*/
     GC            gc;    /* The default graphics context for the window */ 
     GC            pmgc;  /* The default graphics context for the pixmap */
     GC            cloakgc; /* GC used to draw a cloaked ship */
     GC            redgc;   /* GC used to draw red color */
     GC            yellowgc; /* GC used to draw yellow color */
     GC            orangegc; /* GC used to draw orange color */
     GC            greengc; /* GC used to draw green color */
     Window        window; /* The Xlib window */
     unsigned long fg;
     unsigned long bg; /* Forground and background colors */
     int           depth;/*Depth of the display (Depth of 1 means monochrome)*/
     Colormap      cmap;  /* Colormap for the display */
     XColor        yellow;
     XColor        red;
     XColor        orange;
     XColor        green; /* Values of the colors on the 
                                           display*/

  };

#define M_BULLET 0.1
#define MAXPAIRS 50

#endif


























